var documenterSearchIndex = {"docs":
[{"location":"man/BioCCP/#BioCCP","page":"BioCCP","title":"BioCCP","text":"","category":"section"},{"location":"man/BioCCP/#Intro","page":"BioCCP","title":"Intro","text":"","category":"section"},{"location":"man/BioCCP/","page":"BioCCP","title":"BioCCP","text":"BioCCP.jl applies the Coupon Collector's Problem to combinatorial biotechnology, in particular to aid minimum sample size determination of screening experiments. ","category":"page"},{"location":"man/BioCCP/","page":"BioCCP","title":"BioCCP","text":"Modular designs are considered, created by randomly combining r modules from a set of navailable modules. The module probabilities during the generation of the designs are specified by a probability/abundance vector p_vec. Depending on how many complete sets of modules one wants to observe, parameter m can be increased from its default value of 1 to a higher value. ","category":"page"},{"location":"man/BioCCP/","page":"BioCCP","title":"BioCCP","text":"For a specific combinatorial design set-up of interest, a report with results regarding minimum sample sizes can be easily retrieved by using the provided Pluto notebook.","category":"page"},{"location":"man/BioCCP/#Functions","page":"BioCCP","title":"Functions","text":"","category":"section"},{"location":"man/BioCCP/","page":"BioCCP","title":"BioCCP","text":"expectation_minsamplesize\nstd_minsamplesize\nsuccess_probability\nexpectation_fraction_collected\nprob_occurence_module","category":"page"},{"location":"man/BioCCP/#BioCCP.expectation_minsamplesize","page":"BioCCP","title":"BioCCP.expectation_minsamplesize","text":"expectation_minsamplesize(n; p_vec = ones(n), m = 1, r = 1, normalize = true)\n\nCalculates the expected number of designs needed E[T],  the minimum sample size to observe each module at least m times.\n\nn: number of modules in the design space\np_vec: vector with the probabilities or abundances of the different modules\nm: number of complete sets of modules that need to be collected \nr: number of modules per design\nnormalize: if true, normalize p_vec\n\nReferences:\n\nDoumas, A. V., & Papanicolaou, V. G. (2016). The coupon collector’s problem revisited: generalizing the double Dixie cup problem of Newman and Shepp. ESAIM: Probability and Statistics, 20, 367-399.\nBoneh, A., & Hofri, M. (1997). The coupon-collector problem revisited—a survey of engineering problems and computational methods. Stochastic Models, 13(1), 39-66.\n\nExamples\n\njulia> n = 100\njulia> expectation_minsamplesize(n; p_vec = ones(n), m = 1, r = 1, normalize = true)\n519.0\n\n\n\n\n\n","category":"function"},{"location":"man/BioCCP/#BioCCP.std_minsamplesize","page":"BioCCP","title":"BioCCP.std_minsamplesize","text":"std_minsamplesize(n; p_vec = ones(n), m = 1, r = 1, normalize = true)\n\nCalculates the standard deviation on the number of designs needed std[T],  the standard deviation on the minimum sample size to observe each module at least m times.\n\nn: number of modules in the design space\np_vec: vector with the probabilities or abundances of the different modules\nm: number of complete sets of modules that need to be collected \nr: number of modules per design\nnormalize: if true, normalize p_vec\n\nExamples\n\njulia> n = 100\njulia> std_minsamplesize(n; p_vec = ones(n), m = 1, r = 1, normalize = true)\n126.0\n\n\n\n\n\n","category":"function"},{"location":"man/BioCCP/#BioCCP.success_probability","page":"BioCCP","title":"BioCCP.success_probability","text":"success_probability(n, t; p_vec = ones(n), m = 1, r = 1, normalize = true)\n\nCalculates the success probability F(t) = P(T < t) or the chance that  the required number of designs to see each module at least m times is smaller than t.\n\nn: number of modules in design space\nt: sample size/number of designs for which to calculate the success probability \np_vec: vector with the probabilities or abundances of the different modules\nm: number of complete sets of modules that need to be collected \nr: number of modules per design\nnormalize: if true, normalize p_vec\n\nReferences:\n\nBoneh, A., & Hofri, M. (1997). The coupon-collector problem revisited—a survey of engineering problems and computational methods. Stochastic Models, 13(1), 39-66.\n\nExamples\n\njulia> n = 100\njulia> t = 600\njulia> success_probability(n, t; p_vec = ones(n), m = 1, r = 1, normalize = true)\n0.7802171997092149\n\n\n\n\n\n","category":"function"},{"location":"man/BioCCP/#BioCCP.expectation_fraction_collected","page":"BioCCP","title":"BioCCP.expectation_fraction_collected","text":"expectation_fraction_collected(n, t; p_vec = ones(n), r = 1, normalize=true)\n\nCalculates the expected fraction of all modules observed after collecting tdesigns.\n\nn: number of modules in design space\nt: sample size/number of designs for which to calculate the expected fraction of modules observed\np_vec: vector with the probabilities or abundances of the different modules \nr: number of modules per design\nnormalize: if true, normalize p_vec\n\nReferences:\n\nBoneh, A., & Hofri, M. (1997). The coupon-collector problem revisited—a survey of engineering problems and computational methods. Stochastic Models, 13(1), 39-66.\n\nExamples\n\njulia> n = 100\njulia> t = 200\njulia> expectation_fraction_collected(n, t; p_vec = ones(n), r = 1, normalize=true)\n0.8660203251420364\n\n\n\n\n\n","category":"function"},{"location":"man/BioCCP/#BioCCP.prob_occurence_module","page":"BioCCP","title":"BioCCP.prob_occurence_module","text":"prob_occurence_module(p, t, j)\n\nCalculates probability that specific module with module probability p  has occured j times after collecting t designs.\n\nSampling of modules are assumed to be independent Poisson processes.\n\np: module probaility\nt: sample size/number of designs \nj: number of occurence \n\nReferences:\n\nBoneh, A., & Hofri, M. (1997). The coupon-collector problem revisited—a survey of engineering problems and computational methods. Stochastic Models, 13(1), 39-66.\n\nExamples\n\njulia> p = 0.005\njulia> t = 500\njulia> t = 2\njulia> prob_occurence_module(p, t, j)\n0.25651562069968376\n\n\n\n\n\n","category":"function"},{"location":"man/BioCCP/#References","page":"BioCCP","title":"References","text":"","category":"section"},{"location":"man/BioCCP/","page":"BioCCP","title":"BioCCP","text":"Doumas, A. V., & Papanicolaou, V. G. (2016). The coupon collector’s problem revisited: generalizing the double Dixie cup problem of Newman and Shepp. ESAIM: Probability and Statistics, 20, 367-399.\nBoneh, A., & Hofri, M. (1997). The coupon-collector problem revisited—a survey of engineering problems and computational methods. Stochastic Models, 13(1), 39-66.","category":"page"},{"location":"#BioCCP-documentation","page":"BioCCP documentation","title":"BioCCP documentation","text":"","category":"section"},{"location":"","page":"BioCCP documentation","title":"BioCCP documentation","text":"This is the documentation page for the BioCCP.  ","category":"page"},{"location":"","page":"BioCCP documentation","title":"BioCCP documentation","text":"","category":"page"}]
}
